/* guileobjc.h - Interface for glue between Guile and Objective-C
   Copyright (C) 1995 R. Andrew McCallum

   Written by:  R. Andrew McCallum <mccallum@gnu.ai.mit.edu>
   Date: April 1995

   This file is part of the GUILE/Objective-C Library.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
*/

/* Masatake YAMATO (masata-y@is.aist-nara.ac.jp) adapted 
   for working with guile-1.2 and NSObject(libobjects-0.1.19). */

#ifndef __guileobjc_h_INCLUDE
#define __guileobjc_h_INCLUDE 

#include <objc/Object.h>
#include <Foundation/NSObject.h>
#include <Foundation/NSString.h>
/* In the declaration of `scm_setuid@guile-1.[0-2]/libguile/posix.h',
   `id', an objc key word is used as a parameter name.
   To avoid syntax error, I put `define' and `undef' instructions 
   around the next `include' instruction. */
#define id id0 
# include <libguile.h>
# include <guile/gh.h>
#undef id
#define id id

#include <goconfig.h>

#if !defined(HAVE_SCM_PUTS)
#if defined(HAVE_SCM_GEN_PUTS)
# define scm_puts(STR, PORT)	scm_gen_puts (scm_regular_string, STR, PORT)
# define scm_putc(C, PORT)	scm_gen_putc (C, PORT)
#else
# error dont know how to print scm
#endif
#endif

#define RESPONDS_TO	respondsTo	/* FIXME: should be configured */

/* Get this library version */
extern const char * guileobjc_version();
extern SCM guileobjc_version_fn();

/* A table contains print, mark, free, equal functions.
   The guile uses these functions to handle an objc id. */
extern struct scm_smobfuns guileobjc_id_smob; 

/* Key to select `guileobjc_id_smob' in the smon tables.
   Each scm objects which contains an objc id in its CDR slot,
   contains this integer value in its CAR slot. */
extern int scm_tc16_objc_id;

/* Initialize the GUILE/Objective-C interface. */
extern void guileobjc_init();

/* Make new SCM for id */
extern SCM guileobjc_make_objc_id(id o, BOOL allocated, BOOL autoreleased);

/* Get the id from a SCM */
extern id guileobjc_scm2id(SCM o);

/* Lookup a protocol by its name */
extern Protocol * objc_lookup_protocol(char * name);

@protocol GuileObjc
- (void) printForGuile: (SCM)port;
- (const char*)guileObjcName;
@end

@interface Object (GuileObjc)<GuileObjc>
/* This method responsible for what goes in the #<...> when
   Scheme prints the object. */
- (void) printForGuile: (SCM)port;
- (const char*)guileObjcName;
@end

@interface NSObject (GuileObjc)<GuileObjc>
- (void) printForGuile: (SCM)port;
- (const char*)guileObjcName; 
@end

/* Give value to the next pointer to a function, 
   and you can replace default NSString print method with your own function. */
extern void (*NSString_print_for_guile)(NSString * obj, SEL sel, SCM port);

@interface NSString (GuileObjc)
- (void) printForGuile: (SCM)port;
@end

#endif /* __guileobjc_h_INCLUDE */
